#' Function to determine length of a file in hours
#'
#' This function allows the user to determine the length of a file
#' based on the datetime values in the format %Y%M%D%H%m
#' @param length.stay Determined by user to eliminate files that are shorter than the desired length. Defaults to 8 days.
#' @keywords length files
#' @export
#' @examples
#' length_files(length.stay=8)

length_files <- function(){
  
  #lockEnvironment(.Globalenv,FALSE)
  
  ###This function allows the user to determine the length of stay for each file.
  ###A length.stay is determined by user to eliminate files that are shorter than
  ###the desired length of stay. (Default: length.stay = 8 days).
  
  ###Required packages:
  ## lubridate
  
  durations <- c()
  
  for(i in recordnamelist){
    tempfile <- get(i)
    fileduration <- as.numeric(as.duration(interval(ymd_hm(tempfile[1,1]),ymd_hm(tempfile[nrow(tempfile),1]))))
    fileduration <- (fileduration/3600)/24
    durations <- append(durations,fileduration,after=length(durations))
  }
  
  filelengths <<- data.frame(recordnamelist,durations)
  
  write.csv(filelengths,"lengths of stay.csv",row.names=TRUE)
  
  windows()
  
  #hist(filelengths$durations,xlab="Duration (days)",main="Lengths of stay for each record")
  days_dur_crit()
  
  varEntryDialog <- function(vars, 
                             labels = vars,
                             fun = rep(list(as.character), length(vars)),
                             title = 'Variable Entry',
                             prompt = NULL) {
    
    
    stopifnot(length(vars) == length(labels), length(labels) == length(fun))
    
    # Create a variable to keep track of the state of the dialog window:
    # done = 0; If the window is active
    # done = 1; If the window has been closed using the OK button
    # done = 2; If the window has been closed using the Cancel button or destroyed
    done <- tclVar(0)
    
    tt <- tktoplevel()
    tkwm.title(tt, title)	
    entries <- list()
    tclvars <- list()
    
    # Capture the event "Destroy" (e.g. Alt-F4 in Windows) and when this happens, 
    # assign 2 to done.
    tkbind(tt,"<Destroy>",function() tclvalue(done)<-2)
    
    for(i in seq_along(vars)) {
      tclvars[[i]] <- tclVar("")
      entries[[i]] <- tkentry(tt, textvariable=tclvars[[i]])
    }
    
    doneVal <- as.integer(tclvalue(done))
    results <- list()
    
    reset <- function() {
      for(i in seq_along(entries)) {
        tclvalue(tclvars[[i]]) <<- ""
      }
    }
    reset.but <- tkbutton(tt, text="Reset", command=reset)
    
    cancel <- function() {
      tclvalue(done) <- 2
    }
    cancel.but <- tkbutton(tt, text='Cancel', command=cancel)
    
    submit <- function() {
      for(i in seq_along(vars)) {
        tryCatch( {
          results[[vars[[i]]]] <<- fun[[i]](tclvalue(tclvars[[i]]))
          tclvalue(done) <- 1
        },
        error = function(e) { tkmessageBox(message=geterrmessage()) },
        finally = { }
        )
      }
    }
    submit.but <- tkbutton(tt, text="Submit", command=submit)
    
    if(!is.null(prompt)) {
      tkgrid(tklabel(tt,text=prompt), columnspan=3, pady=10)
    }
    
    for(i in seq_along(vars)) {
      tkgrid(tklabel(tt, text=labels[i]), entries[[i]], pady=10, padx=10, columnspan=4)
    }
    
    tkgrid(submit.but, cancel.but, reset.but, pady=10, padx=10, columnspan=3)
    tkfocus(tt)
    
    # Do not proceed with the following code until the variable done is non-zero.
    #   (But other processes can still run, i.e. the system is not frozen.)
    tkwait.variable(done)
    
    if(tclvalue(done) != 1) {
      results <- NULL
    }
    
    tkdestroy(tt)
    return(results)
  }
  
  
  
  length.stay <- as.numeric(varEntryDialog(vars=c('Var1'), labels=c('Based on the presented plot.\nWhat is the minimum length of stay?'), fun=c(
    function(x) {
      x <- as.numeric(x)
      # if(x >= 0 & x <= 10) {
      #   return(x)
      # } else {
      #   stop("Why didn't you follow instruction!\nPlease enter a number between 0 and 10.")
      # }
    } )))
  
  shorter.length.stay <- which(filelengths[,2]<length.stay)
  
  recordnamelist <<- recordnamelist[-c(shorter.length.stay)]
  
  rm(list= c(as.character(filelengths[c(shorter.length.stay),1])),inherits=TRUE)
}
