require(lubridate)
require(stringr)
require(CATkit)
require(zoo)
require(nnet)
require(lomb)
require(signal)
require(mFilter)


options(scipen=999)

filePath <- dirname(file.choose())

setwd(filePath)

filename <- file.choose()

filename <- gsub("\\","/",filename,fixed=TRUE)

filename <- gsub(paste(filePath,"/",sep=""),"",filename,fixed=TRUE)

fileName <- file.path(filePath,filename)

sensor <- read.csv(fileName,header=TRUE)

date.time <- str_split_fixed(sensor[,1]," ",n=2)

dates <- str_split_fixed(date.time[,1],"/",n=3)
dates[,1] <- formatC(as.numeric(as.character(dates[,1])),width=2,format="d",flag="0")
dates[,2] <- formatC(as.numeric(as.character(dates[,2])),width=2,format="d",flag="0")
dates[,3] <- as.character(dates[,3])

times <- str_split_fixed(date.time[,2],":",n=2)
times[,1] <- formatC(as.numeric(as.character(times[,1])),width=2,format="d",flag="0")
times[,2] <- formatC(as.numeric(as.character(times[,2])),width=2,format="d",flag="0")

datetime <- paste(dates[,3],dates[,1],dates[,2],times[,1],times[,2],sep="")

sensor[,1] <- NULL

sensor <- cbind(datetime,sensor)

rm(date.time)
rm(dates)
rm(times)
rm(datetime)

filename <- gsub(".csv","formatdate.csv",filename)

fileName <- file.path(filePath,filename)

write.csv(x=sensor,file=filename,row.names=FALSE)

databin30 <- matrix()


pvalues <- list()
periods <- list()
powers <- list()
cospvals <- list()
cosphis <- list()
cosphises <- list()
cosamps <- list()
cosampses <- list()

Valcols <- 2
  
  
  
  #seq(from=2,to=ncol(sensor),by=1)
TimeCol=1
endX <- 2
p.alpha <- 0.05
for(j in Valcols){
  myHeader <- read.table(fileName, header = TRUE, nrows = 1,
                         sep = ",", skip = 0)
  myData1A <- read.table(fileName, header = TRUE, sep = ",", 
                         skip = 0, stringsAsFactors = FALSE, comment.char = "", 
                         col.names = names(myHeader))
  myData1 <- myData1A[, c(TimeCol = TimeCol, valCols = j)]
  myData1[, TimeCol[1]] <- as.POSIXct(strptime(x = paste(myData1[,TimeCol[1]], myData1[, TimeCol[1]]), format = "%Y%m%d%H%M%S" , tz = "GMT"))      #Changes date format.
  myList <- do.call(Import, list(x = myData1, tsCol = 1, valCols = 2, sumCols = 1,
                                  rowDurationSeconds = 1 * 60, maxGap = nrow(myData1), 
                                  verbose = 0, tz = "GMT"))
  test <- myList$df[,1] %in% myData1[,1]
  
  data1min <- myList$df
  
  
  
  for(i in 1:length(test)){
    if(test[i]==FALSE){
      data1min[i,2] = 0
    }
  }
  endtrial <- nrow(data1min)-30
  
  rm(test)
  
  bin30 <- data.frame()
  for(i in seq(from=1,to=endtrial, by=30)){
    bin30 <- rbind(bin30,data1min[i,])
  }
  rm(endtrial)
  
  databin30 <- cbind(databin30,bin30)
  
  CatBins <- list()
  CatBins$binNames <- names(bin30)
  CatBins$binsPerDay <- 48
  CatBins$dataPts <- nrow(bin30)
  CatBins$dayPts <- 48
  CatBins$newMinPerBin <- 30
  CatBins$plotCnt <- ncol(bin30)-1
  
  
  movingwindow <- 7*(1440/CatBins$newMinPerBin)
  
  cntwindows <- floor((CatBins$dataPts)/(1440/CatBins$newMinPerBin))*(1440/CatBins$newMinPerBin)-movingwindow
  startwindows <- seq(from = 0, to = cntwindows, by = movingwindow)
  
  auto <- list()
  
  period <- c()
  power <- c()
  p.value <- c()
  lombout <- list()
  lomboutdata <- list()
  cospval <- c()
  cosphi <- c()
  cosphise <- c()
  cosamp <- c()
  cosampse <- c()
  times <- seq(from=.5,to=7*24,by=0.5)
  
  
  
  for(i in startwindows){
    
    datacos <- bin30[1:movingwindow+i,]
    if(any(datacos[,2])==0){
      days <- movingwindow/(1440/CatBins$newMinPerBin)
      periodNA <- rep(NA,days)
      period <- append(period,periodNA,after=length(period))
      powerNA <- rep(NA,days)
      power <- append(power,powerNA,after=length(power))
      p.valueNA <- rep(NA,days)
      p.value <- append(p.value,p.valueNA,after=length(p.value))
      cospvalNA <- rep(NA,days)
      cospval <- append(cospval,cospvalNA,after=length(cospval))
      cosphiNA <- rep(NA,days)
      cosphi <- append(cosphi,cosphiNA,after=length(cosphi))
      cosphiseNA <- rep(NA,days)
      cosphise <- append(cosphise,cosphiseNA,after=length(cosphise))
      cosampNA <- rep(NA,days)
      cosamp <- append(cosamp,cosampNA,after=length(cosamp))
      cosampseNA <- rep(NA,days)
      cosampse <- append(cosampse,cosampseNA,after=length(cosampse))
      
      next
    }
    datacos[,1] <- gsub("[[:punct:]]","",datacos[,1])
    datacos[,1] <- gsub(" ","",datacos[,1])
    datacos[,1] = substr(datacos[,1],1,nchar(datacos[,1])-2)
    write.csv(datacos,"datacos.csv",row.names=FALSE)
    rm(datacos)
    filecos <- file.path(filePath,"datacos.csv")
    
    CatBins$DataList <- matrix(bin30[1:movingwindow+i,2],nrow=1)
    CatBins$TimeBins <- bin30[1:movingwindow+i,1]
    DataList <- CatBins$DataList
    
    
    
    LagPcnt <- 1
    Avg <- FALSE
    modulo <- 1440
    fileNum <- 1
    
    startData <-1
    TimeBins<-CatBins$TimeBins
    startX<-2    #  time is in column 1
    startDark <-as.character(format(as.POSIXct(TimeBins[startData], origin="1970-01-01",tz="GMT"),"%H:%M:%S"))    #  get index of first darkness value
    thisTime <- format(Sys.time(), "%b %d, %Y - %H:%M:%S");
    nameVector = CatBins$binNames[-1]     # exclude the DateTime header name
    endXrel<-endX-startX+1
    for (cnt in 1:CatBins$plotCnt) {      #  1 plot per animal, for all animals 
      curX<-cnt+startX-1
      MaxLag <- round((CatBins$dataPts/CatBins$dayPts)*LagPcnt)*CatBins$dayPts        #dayPts is really pts in a modulus (chosen period)
      if (MaxLag<CatBins$dayPts){     #  cannot be less than one period
        MaxLag<-CatBins$dayPts
      }
      
      getACF <- acf(DataList[cnt,],main="",type="correlation",ylab="Correlation coefficient (r)",plot = FALSE, lag.max = MaxLag,col = "white")
      yACF <- as.numeric(getACF$acf)
      
      for(q in seq(from=1, to=length(yACF))){
        nantest <- is.nan(yACF[q])
        if(nantest==TRUE){
          yACF[q]=0
        }
      }
      
      auto[[length(auto)+1]] <- yACF
      xACF <- as.numeric(seq(from = 0, to = length(yACF)-1, by = 1))
      # plot(xACF,yACF,type="l",col="blue",ylim=c(-1,1))
      # 
      # abline(v=seq(from=0, to=MaxLag, by=(CatBins$binsPerDay)),xaxt="n",col="darkslategray",lty="dotted")
      # abline(h=0,col="black")
      # mtext(text=paste(CatBins$binNames[2]),side=3,outer=TRUE,padj=1.5)
      if (cnt==1){
        fileMatrix<-getACF$lag
      }
    }
    
    
    
    
    #########
    #########lomb-Scargle of data
    dataraw <- as.vector(DataList)
    lomboutraw <- jon_lsp(dataraw,times=times,from=18,to=30,ofac=5,alpha=p.alpha,type="period",plot=FALSE)
    period <- append(period,lomboutraw$peak.at[1],after=length(period))
    power <- append(power,lomboutraw$peak,after=length(power))
    p.value <- append(p.value,lomboutraw$p.value,after=length(p.value))
    #,from=18,to=30
    #assign(x=paste("lomboutdata",i,sep=""),value=lomboutraw)
    #########lomb-Scargle of data
    per <- lomboutraw$peak.at[1]
    
    if(is.na(per)==TRUE){
      cospval <- append(cospval,NA,after=length(cospval))
      cosphi <- append(cosphi,NA,after=length(cosphi))
      cosphise <- append(cosphise,NA,after=length(cosphise))
      cosamp <- append(cosamp,NA,after=length(cosamp))
      cosampse <- append(cosampse,NA,after=length(cosampse))}
    else{
      # pdf("test.pdf")
      testCosinor(TimeCol=1,Y=2, Components=1, window="noTaper", RefDateTime=NA,
                  timeFormat="%Y%m%d%H%M", RangeDateTime=list(Start=NA,End=NA),
                  Units="Hours", Progressive=list(Interval=0, Increment=0),
                  Period=list(Set=per,Start=0,Increment=1,End=0),header=TRUE,
                  Skip=0, Colors="BW",Graphics="pdf",Output=list(Txt=FALSE,Dat=FALSE,Doc=FALSE,Graphs=FALSE),
                  yLabel="",Console=TRUE,Debug=FALSE,IDcol="filename",fileName=filecos,functionName="")
      file.remove(filecos)
      cospval <- append(cospval,PVAL,after=length(cospval))
      cosphi <- append(cosphi,PHI,after=length(cosphi))
      cosphise <- append(cosphise,PHI.SE,after=length(cosphise))
      cosamp <- append(cosamp,AMP,after=length(cosamp))
      cosampse <- append(cosampse,AMP.SE,after=length(cosampse))
    }
  }
  
  pvalues[[length(pvalues)+1]] <- p.value
  powers[[length(powers)+1]] <- power
  periods[[length(periods)+1]] <- period
  cospvals[[length(cospvals)+1]] <- cospval
  cosphis[[length(cosphis)+1]] <- cosphi
  cosphises[[length(cosphises)+1]] <- cosphise
  cosamps[[length(cosamps)+1]] <- cosamp
  cosampses[[length(cosampses)+1]] <- cosampse
  
}

columns <- names(sensor[,2:length(sensor)])

pvaluesall <- data.frame(matrix(unlist(pvalues),ncol=length(names(sensor))-1,nrow=length(startwindows),byrow=FALSE)) 
colnames(pvaluesall) <- columns
write.csv(pvaluesall,paste(fileName,"pvalues.csv"),row.names = TRUE)



powersall <- data.frame(matrix(unlist(powers),ncol=length(names(sensor))-1,nrow=length(startwindows),byrow=FALSE))
colnames(powersall) <- columns
write.csv(powersall,paste(fileName,"powers.csv"),row.names = TRUE)

periodsall <- data.frame(matrix(unlist(periods),ncol=length(names(sensor))-1,nrow=length(startwindows),byrow=FALSE))
colnames(periodsall) <- columns
write.csv(periodsall,paste(fileName,"periods.csv"),row.names = TRUE)

cospvalsall <- data.frame(matrix(unlist(cospvals),ncol=length(names(sensor))-1,nrow=length(startwindows),byrow=FALSE))
colnames(cospvalsall) <- columns
write.csv(cospvalsall,paste(fileName,"_cospvals.csv",sep=""),row.names = TRUE)

cosphisall <- data.frame(matrix(unlist(cosphis),ncol=length(names(sensor))-1,nrow=length(startwindows),byrow=FALSE))
colnames(cosphisall) <- columns
write.csv(cosphis,paste(fileName,"_cosphis.csv",sep=""),row.names = TRUE)

cosphisesall <- data.frame(matrix(unlist(cosphises),ncol=length(names(sensor))-1,nrow=length(startwindows),byrow=FALSE))
colnames(cosphisesall) <- columns
write.csv(cosphises,paste(fileName,"_cosphises.csv",sep=""),row.names = TRUE)

cosampsall <- data.frame(matrix(unlist(cosamps),ncol=length(names(sensor))-1,nrow=length(startwindows),byrow=FALSE))
colnames(cosampsall) <- columns
write.csv(cosamps,paste(fileName,"_cosamps.csv",sep=""),row.names = TRUE)

cosampsesall <- data.frame(matrix(unlist(cosampses),ncol=length(names(sensor))-1,nrow=length(startwindows),byrow=FALSE))
colnames(cosampsesall) <- columns
write.csv(cosampses,paste(fileName,"_cosampses.csv",sep=""),row.names = TRUE)






#bin30 <- data.frame(trial[1,])






